#!/bin/bash

# Copyright (C) 2021 |Méso|Star> (contact@meso-star.com)
# Copyright (C) 2021 CNRS/RAPSODEE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

if [ "$BASH_VERSION" == "" ]; then
  >&2 echo "This script must be run with Bash, try 'bash $0'"
  exit 1
fi

set -e

data_path=$(dirname "$(realpath "$BASH_SOURCE")")

################################################################################
# User parameters
################################################################################
laser_position="0,-0.1,0.1"     # Position for the laser
laser_target="0,0.1,0.1"        # Target for the laser
laser_up="1,0,0"                # UP direction for the laser sheet
laser_width=0.2                 # Width of the laser sheet [m]
laser_thickness=0.001           # Thickness of the laser sheet [m]
laser_flux_density=1000         # Laser flux density [W/m^2]

image_definition="800x600"      # Number of pixels of the flux density map
image_spp="128"                 # Number of samples per pixel

camera_position="0.18,0,0.15"   # Camera position
camera_target="-0.36,0,0.1"     # Targeted point
camera_up="0,0,1",              # Camera up vector
camera_fov="70"                 # Vertical field of view (degrees)

lambda=532                      # Wavelength [nm]
fractal_dimension=1.80          # Value of the fractal dimension
fractal_prefactor=1.30          # Value of the fractal prefactor

octree_hint=256                 # Hint on the finest octree definition
optical_thickness=1             # Optical thichkness criterion
overwrite_output=0              # Force the overwriting of the output file
use_simd=1                      # Use SIMD instruction sets if available
cache_octree=1                  # Save/reuse the acceleration structure

image_file_name="dummy_image_${image_definition}x${image_spp}.txt"
cache_file_name="dummy_${octree_hint}$([ ! $use_simd -eq 0 ] && echo '_simd' || echo '').cache"

################################################################################
# Build the command line to execute
################################################################################
cmd="htrdr-combustion -v"
cmd+=" -m ${data_path}/tetra_mesh.sth" #
cmd+=" -O dummy_${octree_hint}.cache"
cmd+=" -p ${data_path}/thermodynamic_conditions.atrtp"
cmd+=" -r ${data_path}/refraction_index.atrri"
cmd+=" -T ${optical_thickness}"
cmd+=" -V ${octree_hint}"
cmd+=" -C pos=${camera_position}:tgt=${camera_target}:up="${camera_up}":fov=${camera_fov}"
cmd+=" -i def=${image_definition}:spp=${image_spp}"
cmd+=" -l pos=${laser_position}:tgt=${laser_target}:up=${laser_up}:sz=${laser_thickness},${laser_width}"
cmd+=" -D ${laser_flux_density}"
cmd+=" -w ${lambda}"
cmd+=" -F dimension=${fractal_dimension}:prefactor=${fractal_prefactor}"
cmd+=" -o ${image_file_name}"
[ ! $overwrite_output -eq 0 ] && cmd+=" -f"
[ ! $use_simd -eq 0 ] && cmd+=" -s"
[ ! $cache_octree -eq 0 ] && cmd+=" -O ${cache_file_name}"

################################################################################
# Run the computation
################################################################################
echo $cmd
eval $cmd
