c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine null_collision_Tr(dim,box_volume,
     &     nc_tr_version,x1,x2,prop,weight)
      implicit none
      include 'max.inc'
c     
c     Purpose: frontend to the 2 possible variants (with or without using k_min)
c     
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + nc_tr_version: version of null-collision for evaluating the transmissivity
c       + x1: first position
c       + x2: second position
c       + prop: 3-character string ('abs', 'sca' or 'ext')
c     
c     Output:
c       + weight: MC weight
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      integer nc_tr_version
      double precision x1(1:Ndim_mx)
      double precision x2(1:Ndim_mx)
      character*3 prop
      double precision weight
c     temp
      double precision u(1:Ndim_mx)
      double precision x(1:Ndim_mx)
      double precision tmp(1:Ndim_mx)
      double precision k,k_hat,k_min
      double precision sigma,L
      logical keep_running
      double precision length,pdf_l
      double precision p,r
c     label
      character*(Nchar_mx) label
      label='subroutine null_collision_Tr'

      if (nc_tr_version.eq.1) then
         call null_collision_Tr_max(dim,box_volume,x1,x2,prop,weight)
      else if (nc_tr_version.eq.2) then
         call null_collision_Tr_minmax(dim,box_volume,x1,x2,prop,
     &        weight)
      else
         call error(label)
         write(*,*) 'nc_tr_version=',nc_tr_version
         stop
      endif

      return
      end


      
      subroutine null_collision_Tr_max(dim,box_volume,x1,x2,prop,
     &     weight)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
c     
c     Purpose: to perform a realization for the transmissivity
c     between 2 positions using only the k_max
c     
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + x1: first position
c       + x2: second position
c       + prop: 3-character string ('abs', 'sca' or 'ext')
c     
c     Output:
c       + weight: MC weight
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x1(1:Ndim_mx)
      double precision x2(1:Ndim_mx)
      character*3 prop
      double precision weight
c     temp
      double precision u(1:Ndim_mx)
      double precision x(1:Ndim_mx)
      double precision tmp(1:Ndim_mx)
      double precision k,k_hat
      double precision sigma,L
      logical keep_running
      double precision length,pdf_l
      double precision p,r
c     label
      character*(Nchar_mx) label
      label='subroutine null_collision_Tr_max'

      call substract_vectors(dim,x2,x1,tmp) ! vector x2-x1
      call normalize_vector(dim,tmp,u) ! u: direction from x1 to x2
      call vector_length(dim,tmp,L) ! length of vector x2-x1

      if (trim(prop).eq.'abs') then
         k_hat=ka_hat
      else if (trim(prop).eq.'sca') then
         k_hat=ks_hat
      else if (trim(prop).eq.'ext') then
         k_hat=ka_hat+ks_hat
      else
         call error(label)
         write(*,*) 'Bad input argument:'
         write(*,*) 'prop=',trim(prop)
         write(*,*) 'must be "abs", "sca" or "ext"'
         stop
      endif
      if (k_hat.le.0.0D+0) then
         call error(label)
         write(*,*) 'k_hat=',k_hat
         write(*,*) 'should be positive'
         stop
      endif

      sigma=0.0D+0
      keep_running=.true.
      do while (keep_running)
         call choose_length_unlimited(k_hat,length,pdf_l)
         sigma=sigma+length
         if (sigma.gt.L) then
            weight=1.0D+0
            keep_running=.false.
         else
            call scalar_vector(dim,sigma,u,tmp)
            call add_vectors(dim,x1,tmp,x) ! x: local position at distance 'sigma' from x1 toward x2
            if (trim(prop).eq.'abs') then
               call absorption_coefficient(dim,box_volume,x,k)
            else if (trim(prop).eq.'sca') then
               call scattering_coefficient(dim,box_volume,x,k)
            else if (trim(prop).eq.'ext') then
               call extinction_coefficient(dim,box_volume,x,k)
            endif
            p=k/k_hat ! probability of absorption
            call random_gen(r)
            if (r.le.p) then
               weight=0.0D+0
               keep_running=.false.
            endif               ! value of r
         endif                  ! value of x
      enddo                     ! while (keep_running)
      
      return
      end


      
      subroutine null_collision_Tr_minmax(dim,box_volume,x1,x2,prop,
     &     weight)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
c     
c     Purpose: to perform a realization for the transmissivity
c     between 2 positions using both a k_min and a k_max
c     
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + x1: first position
c       + x2: second position
c       + prop: 3-character string ('abs', 'sca' or 'ext')
c     
c     Output:
c       + weight: MC weight
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x1(1:Ndim_mx)
      double precision x2(1:Ndim_mx)
      character*3 prop
      double precision weight
c     temp
      double precision u(1:Ndim_mx)
      double precision x(1:Ndim_mx)
      double precision tmp(1:Ndim_mx)
      double precision k,k_hat,k_min
      double precision sigma,L
      logical keep_running
      double precision length,pdf_l
      double precision p,r
c     label
      character*(Nchar_mx) label
      label='subroutine null_collision_Tr_minmax'

      call substract_vectors(dim,x2,x1,tmp) ! vector x2-x1
      call normalize_vector(dim,tmp,u) ! u: direction from x1 to x2
      call vector_length(dim,tmp,L) ! length of vector x2-x1

      if (trim(prop).eq.'abs') then
         k_min=ka_min
         k_hat=ka_hat
      else if (trim(prop).eq.'sca') then
         k_min=ks_min
         k_hat=ks_hat
      else if (trim(prop).eq.'ext') then
         k_min=ka_min+ks_min
         k_hat=ka_hat+ks_hat
      else
         call error(label)
         write(*,*) 'Bad input argument:'
         write(*,*) 'prop=',trim(prop)
         write(*,*) 'must be "abs", "sca" or "ext"'
         stop
      endif
      if (k_hat.le.0.0D+0) then
         call error(label)
         write(*,*) 'k_hat=',k_hat
         write(*,*) 'should be positive'
         stop
      endif

      sigma=0.0D+0
      keep_running=.true.
      do while (keep_running)
         call choose_length_unlimited(k_hat-k_min,length,pdf_l)
         sigma=sigma+length
         if (sigma.gt.L) then
            weight=dexp(-k_min*L)
            keep_running=.false.
         else
            call scalar_vector(dim,sigma,u,tmp)
            call add_vectors(dim,x1,tmp,x) ! x: local position at distance 'sigma' from x1 toward x2
            if (trim(prop).eq.'abs') then
               call absorption_coefficient(dim,box_volume,x,k)
            else if (trim(prop).eq.'sca') then
               call scattering_coefficient(dim,box_volume,x,k)
            else if (trim(prop).eq.'ext') then
               call extinction_coefficient(dim,box_volume,x,k)
            endif
            p=(k-k_min)/(k_hat-k_min) ! probability of absorption
            call random_gen(r)
            if (r.le.p) then
               weight=0.0D+0
               keep_running=.false.
            endif               ! value of r
         endif                  ! value of x
      enddo                     ! while (keep_running)
      
      return
      end



      subroutine mc_transmissivity(dim,box_volume,nc_tr_version,
     &     x1,x2,prop,Nevent)
      implicit none
      include 'max.inc'
c     
c     Purpose: to test the "null_collision_Tr" subroutine in a homogeneous medium
c     
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + nc_tr_version: version of null-collision for evaluating the transmissivity
c       + x1: first position
c       + x2: second position
c       + prop: 3-character string ('abs', 'sca' or 'ext')
c     
c     Output:
c       + on-screen: results
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      integer nc_tr_version
      double precision x1(1:Ndim_mx)
      double precision x2(1:Ndim_mx)
      character*3 prop
      integer Nevent
c     temp
      double precision tmp(1:Ndim_mx)
      double precision k,L,Tr
      integer event
      double precision weight
      double precision sum,sum2
      double precision mean,variance,std_dev
c     label
      character*(Nchar_mx) label
      label='subroutine mc_transmissivity'

c     analytic result
      if (trim(prop).eq.'abs') then
         call absorption_coefficient(dim,box_volume,x1,k)
      else if (trim(prop).eq.'sca') then
         call scattering_coefficient(dim,box_volume,x1,k)
      else if (trim(prop).eq.'ext') then
         call extinction_coefficient(dim,box_volume,x1,k)
      else
         call error(label)
         write(*,*) 'Bad input argument:'
         write(*,*) 'prop=',trim(prop)
         write(*,*) 'must be "abs", "sca" or "ext"'
         stop
      endif
      call substract_vectors(dim,x2,x1,tmp) ! vector x2-x1
      call vector_length(dim,tmp,L) ! length of vector x2-x1
      Tr=dexp(-k*L)

c     MC computation
      sum=0.0D+0
      sum2=0.0D+0
      do event=1,Nevent
         call null_collision_Tr(dim,box_volume,nc_tr_version,x1,x2,
     &        prop,weight)
         sum=sum+weight
         sum2=sum2+weight**2.0D+0
      enddo                     ! event
      call statistics(Nevent,sum,sum2,mean,variance,std_dev)

c     Display results
      write(*,*) 'MC result:',mean,' +/-',std_dev
      write(*,*) 'analytic result:',Tr
      
      return
      end
