c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine rdgfa_init(m,kf,Dp,Np,norm_factor)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'rdgfa.inc'
      include 'com_properties.inc'
c     
c     Purpose: to perform initialization tasks related to
c     the RDG-FA model
c     
c     Input:
c       + lambda: wavelength [nm] (via 'com_properties.inc')
c       + m: refractive index [complex]
c       + Df: fractal dimension [] (via 'com_properties.inc')
c       + kf: prefactor []
c       + Dp: primary particles diameter [nm]
c       + Np: number of primary particles in the aggregate
c     
c     Output:
c       + data stored into the "rdgfa" common
c       + norm_factor: RDG-FA phase function normalization factor
c     
c     I/O
      complex m
      double precision kf
      double precision Dp
      double precision Np
      double precision norm_factor
c     temp
      integer i
      double precision xp,k,q,kRg,qRg,Em,Fm
      double precision rho_max
      double precision c,delta_c
      double precision theta1,theta2,dtheta,delta_omega
      double precision phi1,phi2
c     label
      character*(Nchar_mx) label
      label='subroutine rdgfa_init'

c     Number of intervals for the phase function
      Nint=1000
      if (Nint.gt.Nint_mx) then
         call error(label)
         write(*,*) 'Nint=',Nint
         write(*,*) '> Nint_mx=',Nint_mx
         stop
      endif
      
      call gyration_radius(Np,kf,Df,Dp,Rg)
      call function_CapitalE(m,Em)
      call function_CapitalF(m,Fm)
      call wavenumber(lambda,k)
      kRg=k*Rg
      call function_g(Df,kRg,g)
      xp=pi*Dp/lambda

c     Initialization of parameters required to compute ka and ks
      Cabs=4.0D+0*pi*Np*(xp**3.0D+0)*Em/(k**2.0D+0) ! [nm²/aggregate]
      Csca=8.0D+0*pi*(Np**2.0D+0)*(xp**6.0D+0)*Fm*g/(3.0D+0*k**2.0D+0) ! [nm²/aggregate]
      Va=pi*(Dp**3.0D+0)*Np/6.0D+0 ! [nm³ of soot/aggregate]

c     Initialization of ka_min and ks_min
      ka_min=0.0D+0
      ks_min=0.0D+0
      
c     Initialization of ka_hat and ks_hat
      rho_max=fv_max/Va         ! [nm³ of soot / nm³]/[nm³ of soot/aggregate]=[aggregates/nm³]
      ka_hat=1.0D9*rho_max*Cabs ! [m⁻¹]
      ks_hat=1.0D9*rho_max*Csca ! [m⁻¹]

c     Computation of the cumulative of the phase function
      dtheta=pi/dble(Nint)
      c=0.0D+0
      pf_cumulative(0)=0.0D+0
      norm_factor=1.0D+0
      do i=1,Nint
         theta1=dtheta*(dble(i-1))
         theta2=dtheta*(dble(i))
         delta_omega=2.0D+0*pi*dsin((theta1+theta2)/2.0D+0)*dtheta
         call RDGFA_phase_function(dcos(theta1),norm_factor,phi1)
         call RDGFA_phase_function(dcos(theta2),norm_factor,phi2)
         delta_c=(phi1+phi2)*delta_omega/2.0D+0
         c=c+delta_c
         pf_cumulative(i)=c
      enddo                     ! i
      norm_factor=c
      do i=1,Nint
         pf_cumulative(i)=pf_cumulative(i)/norm_factor
      enddo                     ! i

c     Debug
c$$$      open(10,file='./results/rdgfa_phase_function.txt')
c$$$      do i=0,Nint
c$$$         theta2=dtheta*(dble(i))
c$$$         call RDGFA_phase_function(dcos(theta2),norm_factor,phi2)
c$$$         write(10,*) theta2,phi2
c$$$      enddo                     ! i
c$$$      close(10)
c     Debug

      
      return
      end
      


      subroutine gyration_radius(Np,kf,Df,Dp,Rg)
      implicit none
      include 'max.inc'
c         
c     Purpose: to compute the gyration radius
c     
c     Input:
c      + Np: number of primary particles in the aggregate
c      + kf: prefactor
c      + Df: fractal dimension
c      + Dp: primary particles diameter [nm]
c     
c     Output:
c      + Rg: gyration radius [nm]
c     
c     I/O
      double precision Np
      double precision kf
      double precision Df
      double precision Dp
      double precision Rg
c     temp
c     label
      character*(Nchar_mx) label
      label='subroutine gyration_radius'

      Rg=Dp/2.0D+0*(dble(Np)/kf)**(1.0D+0/Df)
      
      return
      end
      


      subroutine wavenumber(lambda,k)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: compute the wavenumber k
c     
c     Input:
c      + lambda: wavelength [nm]
c     
c     Output:
c     + k: wavenumber [nm⁻1]
c     
c     I/O
      double precision lambda
      double precision k
c     temp
c     label
      character*(Nchar_mx) label
      label='subroutine wavenumber'

      k=2.0D+0*pi/lambda
      
      return
      end



      subroutine q_factor(lambda,theta,q)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: compute the q factor
c     
c     Input:
c      + lambda: wavelength [nm]
c      + theta: scattering angle [rad]
c     
c     Output:
c     + q: q factor
c     
c     I/O
      double precision lambda
      double precision theta
      double precision q
c     temp
c     label
      character*(Nchar_mx) label
      label='subroutine q_factor'

      q=4.0D+0*pi/lambda*dsin(theta/2.0D+0)
      
      return
      end



      subroutine function_f(Df,qRg,f)
      implicit none
      include 'max.inc'
c     
c     Purpose: compute the f(qRg) function
c     
c     Input:
c      + Df: fractal dimension
c      + qRg: value of q.Rg
c     
c     Output:
c      + f: value of the f function
c     
c     I/O
      double precision Df
      double precision qRg
      double precision f
c     temp
c     label
      character*(Nchar_mx) label
      label='subroutine function_f'

      if (qRg**2.0D+0.le.1.50D+0*Df) then
         f=dexp(-1.0D+0/3.0D+0*qRg**2.0D+0)
      else
         f=(3.0D+0*Df/(2.0D+0*dexp(1.0D+0)*qRg**2.0D+0))**(Df/2.0D+0)
      endif
      
      return
      end



      subroutine function_g(Df,kRg,g)
      implicit none
      include 'max.inc'
c     
c     Purpose: compute the g(kRg) function
c     
c     Input:
c      + Df: fractal dimension
c      + kRg: value of k.Rg
c     
c     Output:
c      + g: value of the g function
c     
c     I/O
      double precision Df
      double precision kRg
      double precision g
c     temp
c     label
      character*(Nchar_mx) label
      label='subroutine function_g'

      g=(1.0D+0+(4.0D+0*kRg**2.0D+0)/(3.0D+0*Df))**(-0.5D+0*Df)
      
      return
      end



      subroutine function_CapitalE(m,em)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute the E(m) function
c     
c     Input:
c      + m: complex refraction index
c     
c     Ouptut:
c     + em: value of E(m)
c     
c     I/O
      complex m
      double precision em
c     temp
      complex a
c     label
      character*(Nchar_mx) label
      label='subroutine function_CapitalE'

      a=(m**2.0D+0-1.0D+0)/(m**2.0D+0+2.0D+0)
      em=aimag(a)

      return
      end



      subroutine function_CapitalF(m,fm)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute the F(m) function
c     
c     Input:
c      + m: complex refraction index
c     
c     Ouptut:
c     + fm: value of F(m)
c     
c     I/O
      complex m
      double precision fm
c     temp
      complex a
c     label
      character*(Nchar_mx) label
      label='subroutine function_CapitalF'

      a=(m**2.0D+0-1.0D+0)/(m**2.0D+0+2.0D+0)
      fm=cabs(a)**2.0D+0
      
      return
      end 
