c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_data(dim,datafile,box_volume,laser_volume,
     &     emission_surface,integration_surface,d,
     &     phi0,m,kf,Dp,Np,
     &     pf,g_HG,direct_computation,reverse_computation,Nevent)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
      include 'formats.inc'
c     
c     Purpose: to read the "datafile" input file
c     
c     Input:
c       + dim: dimension of space
c       + datafile: input data file to read
c     
c     Output:
c       + box_volume: volume limits of the surrounding box
c       + laser_volume: volume limits of the laser sheet
c       + emission_surface: surface limits of the emission surface
c       + integration_surface: surface limits of the integration surface
c       + d: distance between laser sheet and integration surface [m]
c       + phi0: Laser surface density power [W/m²]
c       + lambda: wavelength [nm] (into 'com_properties.inc')
c       + m: refractive index [complex]
c       + Df: fractal dimension [] (into 'com_properties.inc')
c       + kf: prefactor []
c       + Dp: primary particles diameter [nm]
c       + Np: number of primary particles in the aggregate
c       + fv_max: maximum soot volumic fraction [m³/m³] (into 'com_properties.inc')
c       + pf: choice of phase function [0: Rayleigh, 1:isotropic, 2: Henyey-Greenstein, 3: RDG-FA]
c       + g_HG: value of the asymetry parameter when using a Henyey-Greenstein phase function
c       + direct_computation: T if direct Monte-Carlo integration is required
c       + reverse_computation: T if reverse Monte-Carlo integration is required
c       + Nevent: number of statistical realization for MC
c     
c     I/O
      integer dim
      character*(Nchar_mx) datafile
      double precision box_volume(1:Ndim_mx,1:2)
      double precision laser_volume(1:Ndim_mx,1:2)
      double precision emission_surface(1:Ndim_mx,1:2)
      double precision integration_surface(1:Ndim_mx,1:2)
      double precision d
      double precision phi0
      complex m
      double precision kf
      double precision Dp
      double precision Np
      integer pf
      double precision g_HG
      logical direct_computation
      logical reverse_computation
      integer Nevent
c     temp
      integer i,ios
      logical is_int
      double precision bdim(1:Ndim_mx)
      double precision ldim(1:Ndim_mx)
      double precision sdim(1:Ndim_mx)
c     label
      character*(Nchar_mx) label
      label='subroutine read_data'

      open(10,file=trim(datafile),status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) trim(datafile)
         stop
      endif
      do i=1,39
         read(10,*)
      enddo
      read(10,*) bdim(1)
      read(10,*)
      read(10,*) bdim(2)
      read(10,*)
      read(10,*) bdim(3)
      read(10,*)
      read(10,*) ldim(1)
      read(10,*)
      read(10,*) ldim(2)
      read(10,*)
      read(10,*) ldim(3)
      read(10,*)
      read(10,*) sdim(1)
      read(10,*)
      read(10,*) sdim(2)
      read(10,*)
      read(10,*) d
      do i=1,6
         read(10,*)
      enddo
      read(10,*) phi0
      read(10,*) 
      read(10,*) lambda
      read(10,*) 
      read(10,*) m
      read(10,*) 
      read(10,*) Df
      read(10,*) 
      read(10,*) kf
      read(10,*) 
      read(10,*) Dp
      read(10,*) 
      read(10,*) Np
      read(10,*) 
      read(10,*) fv_max
      do i=1,6
         read(10,*)
      enddo
      read(10,*) pf
      read(10,*)
      read(10,*) g_HG
      read(10,*)
      read(10,*) direct_computation
      read(10,*)
      read(10,*) reverse_computation
      read(10,*)
      read(10,*) Nevent
      close(10)

      sdim(3)=0.0D+0
      do i=1,dim
         if (bdim(i).le.0.0D+0) then
            call error(label)
            write(*,*) 'Incorrect value:'
            write(*,*) 'bdim(',i,')=',bdim(i)
            write(*,*) 'should be > 0'
            stop
         endif
      enddo                     ! i
      do i=1,dim
         if (ldim(i).le.0.0D+0) then
            call error(label)
            write(*,*) 'Incorrect value:'
            write(*,*) 'ldim(',i,')=',ldim(i)
            write(*,*) 'should be > 0'
            stop
         endif
      enddo                     ! i
      do i=1,dim-1
         if (sdim(i).le.0.0D+0) then
            call error(label)
            write(*,*) 'Incorrect value:'
            write(*,*) 'sdim(',i,')=',sdim(i)
            write(*,*) 'should be > 0'
            stop
         endif
      enddo                     ! i
      if (d.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'd=',d
         write(*,*) 'should be > 0'
         stop
      endif
      if (phi0.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'phi0=',phi0
         write(*,*) 'should be > 0'
         stop
      endif
      if (lambda.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'lambda=',lambda
         write(*,*) 'should be > 0'
         stop
      endif
      if (real(m).le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Re(m)=',real(m)
         write(*,*) 'should be > 0'
         stop
      endif
      if (aimag(m).le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Im(m)=',aimag(m)
         write(*,*) 'should be > 0'
         stop
      endif
      if (Df.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Df=',Df
         write(*,*) 'should be > 0'
         stop
      endif
      if (kf.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'kf=',kf
         write(*,*) 'should be > 0'
         stop
      endif
      if (Dp.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Dp=',Dp
         write(*,*) 'should be > 0'
         stop
      endif
      if (Np.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Np=',Np
         write(*,*) 'should be > 0'
         stop
      endif
      if (fv_max.le.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'fv_max=',fv_max
         write(*,*) 'should be > 0'
         stop
      endif
      if ((pf.lt.0).or.(pf.gt.3)) then
         call error(label)
         write(*,*) 'pf=',pf
         write(*,*) 'should be in the [0,3] range'
         stop
      endif
      if ((pf.eq.2).and.((g_HG.lt.-1.0D+0).or.(g_HG.gt.1.0D+0))) then
         call error(label)
         write(*,*) 'g_HG=',g_HG
         write(*,*) 'should be in the [-1,1] range'
         stop
      endif
      if (Nevent.le.0) then
         call error(label)
         write(*,*) 'Incorrect value:'
         write(*,*) 'Nevent=',Nevent
         write(*,*) 'should be > 0'
         stop
      endif
      
c     volume limits of the surrounding box
      box_volume(1,1)=-bdim(1)/2.0D+0
      box_volume(1,2)=bdim(1)/2.0D+0
      box_volume(2,1)=0.0D+0
      box_volume(2,2)=bdim(2)
      box_volume(3,1)=0.0D+0
      box_volume(3,2)=bdim(3)
c     volume limits of the laser sheet
      laser_volume(1,1)=-ldim(1)/2.0D+0
      laser_volume(1,2)=ldim(1)/2.0D+0
      laser_volume(2,1)=0.0D+0
      laser_volume(2,2)=ldim(2)
      laser_volume(3,1)=d
      laser_volume(3,2)=d+ldim(3)
c     surface limits of the emission surface:
c     laser sheet dimensions, z=d
      emission_surface(1,1)=-ldim(1)/2.0D+0
      emission_surface(1,2)=ldim(1)/2.0D+0
      emission_surface(2,1)=0.0D+0
      emission_surface(2,2)=0.0D+0
      emission_surface(3,1)=d
      emission_surface(3,2)=d+ldim(3)
c     surface limits of the integration surface:
c     centered on the center of the laser sheet, z=0
      integration_surface(1,1)=-sdim(1)/2.0D+0
      integration_surface(1,2)=sdim(1)/2.0D+0
      integration_surface(2,1)=(ldim(2)-sdim(2))/2.0D+0
      integration_surface(2,2)=(ldim(2)+sdim(2))/2.0D+0
      integration_surface(3,1)=0.0D+0
      integration_surface(3,2)=0.0D+0+sdim(3)
      
      return
      end
