c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine sw_reverse(dim,box_volume,laser_volume,
     &     emission_surface,integration_surface,
     &     d,phi0,nc_tr_version,pf,g_HG,norm_factor,Nevent)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'com_properties.inc'
c     
c     Purpose: to perform the computation of the flux incoming on the
c     integration surface using a reverse MC algorithm
c     
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + laser_volume: volume limits of the laser sheet
c       + emission_surface: surface limits of the emission surface
c       + integration_surface: surface limits of the integration surface
c       + d: distance between laser sheet and integration surface [m]
c       + phi0: Laser surface density power [W/m²]
c       + nc_tr_version: version of null-collision for evaluating the transmissivity
c       + pf: choice of phase function [0: Rayleigh, 1:isotropic, 2: Henyey-Greenstein, 3: RDG-FA]
c       + g_HG: value of the asymetry parameter when using a Henyey-Greenstein phase function
c       + norm_factor: normalization factor (so that the integral over 4pi sr is equal to 1)
c       + Nevent: number of statistical events
c     
c     Output:
c       + on-screen: value of the flux [W]
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision laser_volume(1:Ndim_mx,1:2)
      double precision emission_surface(1:Ndim_mx,1:2)
      double precision integration_surface(1:Ndim_mx,1:2)
      double precision d
      double precision phi0
      integer nc_tr_version
      integer pf
      double precision g_HG
      double precision norm_factor
      integer Nevent
c     temp
      integer i,j,event,ivar
      logical debug,check1
      double precision u_star(1:Ndim_mx)
      double precision x0(1:Ndim_mx)
      double precision x1(1:Ndim_mx)
      double precision xl1(1:Ndim_mx)
      double precision xl2(1:Ndim_mx)
      double precision xd1(1:Ndim_mx)
      double precision xs1(1:Ndim_mx)
      double precision xr(1:Ndim_mx)
      double precision u0(1:Ndim_mx)
      double precision minus_u0(1:Ndim_mx)
      double precision u1(1:Ndim_mx)
      double precision n0(1:Ndim_mx)
      double precision tmp(1:Ndim_mx)
      double precision S0,Sdetector,pdf_u0
      double precision laser_thickness
      logical intersection_found
      logical first_intersection_found
      logical second_intersection_found
      double precision x_int(1:Ndim_mx)
      double precision x_int1(1:Ndim_mx)
      double precision x_int2(1:Ndim_mx)
      double precision d_x0xint
      double precision n_int(1:Ndim_mx)
      double precision Ld1
      integer errcode
      integer Nvar
      double precision sum(1:Nvar_mx)
      double precision sum2(1:Nvar_mx)
      double precision contrib(1:Nvar_mx),weight
      double precision mean(1:Nvar_mx)
      double precision variance(1:Nvar_mx)
      double precision std_dev(1:Nvar_mx)
      logical on_surface,compute_Ld1
      integer surface_index1,surface_index2
      double precision prod0,prod,tau_max,tau,pdf_tau
      logical keep_going,keep_sampling,scattering_position_found
      logical x0_is_in_laser
      logical x1_is_in_box,x1_is_in_laser
      double precision l_xl1xd1,l_x0x1,d_in_sheet
      double precision length,pdf_length,sigma
      double precision p,r
      double precision Tr
      logical generate_angle
      double precision alpha0,alpha
      double precision phase_function0,phase_function
      double precision taus_max,taus,pdf_taus
      double precision w1,w2,w3,w4,w5
c     progress display
      integer len,ndone,ntot
      double precision fdone,fdone0
      integer ifdone,pifdone
      character*(Nchar_mx) str,source_str
      character*(Nchar_mx) fmt,fmt0
      integer err
      double precision ks
c     label
      character*(Nchar_mx) label
      label='subroutine sw_reverse'

      Nvar=1
c     ----------------- print status
      len=6
      call num2str(len,str,err)
      if (err.eq.0) then
         fmt0='(a,i'//trim(str)//',a)'
         fmt='(i'//trim(str)//',a)'
      endif
c     ----------------- print status
c     check
      if (Nvar.gt.Nvar_mx) then
         call error(label)
         write(*,*) 'Nvar=',Nvar
         write(*,*) '> Nvar_mx=',Nvar_mx
         stop
      endif
c     Initialization
      do ivar=1,Nvar
         sum(ivar)=0.0D+0
         sum2(ivar)=0.0D+0
      enddo ! ivar
      
c     ----------------- print status
      write(*,*) 'Performing reverse MC computation...'
      write(*,trim(fmt0),advance='no')
     &     'Done:   ',floor(fdone0),' %'
      ndone=0
      ntot=Nevent
      pifdone=0
c     ----------------- print status

      laser_thickness=laser_volume(3,2)-laser_volume(3,1) ! [m]
      u_star(1)=0.0D+0
      u_star(2)=1.0D+0
      u_star(3)=0.0D+0                  ! i
c     emission surface
      S0=(emission_surface(1,2)-emission_surface(1,1))*
     &     (emission_surface(3,2)-emission_surface(3,1)) ! [m²]
c     integration surface
      Sdetector=(integration_surface(1,2)-integration_surface(1,1))*
     &     (integration_surface(2,2)-integration_surface(2,1))
c     normal to the integration surface
      n0(1)=0.0D+0
      n0(2)=0.0D+0
      n0(3)=1.0D+0
c     needed for intersections
      debug=.false.
      check1=.true.
c     
      do event=1,Nevent
c     initialization of the path' transmissivity
         Tr=1.0D+0
c     weight initialization
         do ivar=1,Nvar
            contrib(ivar)=0.0D+0
         enddo                  ! ivar
         weight=0.0D+0
c     sampling the integration surface
         do i=1,dim
            call random_gen(r)
            x0(i)=integration_surface(i,1)
     &           +(integration_surface(i,2)-integration_surface(i,1))*r
         enddo
c     check
         call is_on_surface(dim,x0,box_volume,
     &        on_surface,surface_index1,surface_index2)
         if (.not.on_surface) then
            call error(label)
            write(*,*) 'x0=',x0
            write(*,*) 'is not on the surface of box_volume'
            stop
         endif
c     sampling emission direction
         call choose_emission_direction_on_surface(dim,n0,u0,pdf_u0)
         
c     check
         call scalar_product(dim,u0,n0,prod)
         if (prod.lt.0.0D+0) then
            call error(label)
            write(*,*) 'n0=',n0
            write(*,*) 'u0=',u0
            write(*,*) '|u0.n0|=',prod
            write(*,*) 'should be positive'
            stop
         endif
c         
         keep_going=.true.
         do while (keep_going)
            compute_Ld1=.false. ! default
c     check if x0 is inside the laser sheet
            call is_in_box(dim,x0,laser_volume,x0_is_in_laser)
            if (x0_is_in_laser) then
               compute_Ld1=.true.
               call duplicate_vector(dim,x0,xl1)
               call intersection_with_box_from_inside(debug,
     &              dim,check1,
     &              laser_volume,x0,u0,
     &              intersection_found,x_int,
     &              d_x0xint,n_int,errcode)
               if (.not.intersection_found) then
                  call error(label)
                  write(*,*) 'Intersection not found with '
     &                 //'laser_volume from inside'
                  stop
               endif            ! intersection_found=F
               call duplicate_vector(dim,x_int,xl2)
            else                ! x0_is_in_laser=F
c     check if (x0, u0) intersects the laser sheet
               call intersection_with_box_from_outside(debug,dim,check1,
     &              laser_volume,x0,u0,
     &              first_intersection_found,x_int1,d_x0xint,n_int)
               if (first_intersection_found) then
                  call duplicate_vector(dim,x_int1,xl1)
                  call direct_flight(dim,xl1,u0,1.0D-8,x1)
                  call is_in_box(dim,x1,laser_volume,x1_is_in_laser)
                  if (x1_is_in_laser) then
                     call intersection_with_box_from_inside(debug,
     &                    dim,check1,
     &                    laser_volume,x1,u0,
     &                    second_intersection_found,x_int2,
     &                    d_x0xint,n_int,errcode)
                     if (second_intersection_found) then
                        compute_Ld1=.true.
                        call duplicate_vector(dim,x_int2,xl2)
                     else
                        call error(label)
                        write(*,*) 'Second intersection not found'
     &                       //' with laser_volume'
                        write(*,*) 'x1=',x1
                        write(*,*) 'u0=',u0
                        stop
                     endif      ! second_intersection_found
                  endif         ! x0_is_in_laser
               endif            ! first_intersection_found
            endif               ! x0 is in laser_volume or not
c
            if (compute_Ld1) then
c     compute Ld1, the luminance of the laser, scattered in laser sheet,
c     back to x0 in direction -u0
c     d_in_sheet: distance from x0 to exit position of the laser sheet
               call substract_vectors(dim,xl2,xl1,tmp)
               call vector_length(dim,tmp,d_in_sheet)
c     alpha0: scattering angle u_star -> minus_u0
               call scalar_vector(dim,-1.0D+0,u0,minus_u0)
               call scalar_product(dim,u_star,minus_u0,prod0)
               alpha0=dacos(prod0)
c     -------------------------------------------------------------------------------------------
c     Solving emission of the [xl1, xl2] segment using null collisions: w3
c     There is no use of the "nc_tr_version=2" option: we are trying to evaluate
c     a emissivity; therefore we are trying to sample as often as possible
c     within the column: collision lengths have to be minimized, which means
c     the coefficient used to sample collision lengths has to be maximized.
c     -------------------------------------------------------------------------------------------
c     
c     value of the phase function from u_star to -u0 (needed for evaluation of Ld1)
               generate_angle=.false. ! generate scattering angle
               call choose_scattering_angle(dim,
     &              generate_angle,
     &              pf,g_HG,norm_factor,
     &              alpha0,phase_function0)
c     w4: transmissivity due to extinction between x0 and xl1
               call null_collision_Tr(dim,box_volume,nc_tr_version,
     &              x0,xl1,'ext',w4)
c     Initialization
               Ld1=0.0D+0
               w3=1.0D+0
               taus_max=ks_hat*d_in_sheet
               sigma=0.0D+0
               keep_sampling=.true.
               do while (keep_sampling)
                  call choose_tau_limited(taus_max,taus,pdf_taus)
                  length=taus/ks_hat
                  sigma=sigma+length
                  if (sigma.gt.d_in_sheet) then
                     keep_sampling=.false.
                  else
c     xd1: scattering position within the laser sheet
                     call direct_flight(dim,xl1,u0,sigma,xd1)
                     call scattering_coefficient(dim,box_volume,xd1,ks)
c     sampling emission @ xd1 or null-collision
                     call random_gen(r)
                     if (r.le.ks/ks_hat) then ! emission @ xd1
c     xs1 is the projection of xd1 on the emission surface
                        xs1(1)=xd1(1)
                        xs1(2)=laser_volume(2,1)
                        xs1(3)=xd1(3)
c     w1: transmissivty due to extinction between xd1 and xs1
                        call null_collision_Tr(dim,box_volume,
     &                       nc_tr_version,
     &                       xd1,xs1,'ext',w1)
c     w2: transmissivity due to absorption between xl1 and xd1
                        call null_collision_Tr(dim,box_volume,
     &                       nc_tr_version,
     &                       xl1,xd1,'abs',w2)
c     w3: emissivity due to scattering of the [xl1,xl2] segment
                        w3=w3*(1.0D+0-dexp(-taus_max))
                        Ld1=phi0*w1*w2*phase_function0*w3*w4
                        keep_sampling=.false.
                     else ! null-collision @ xd1
                        w3=w3*(1.0D+0-dexp(-taus_max))
                     endif      ! r<p
                  endif         ! value of sigma against d_in_sheet
               enddo            ! while (keep_sampling)
               weight=weight+Ld1*Tr
            endif               ! compute_Ld1
c     -------------------------------------------------------------------------------------------

c     Now a new scattering position has to be sampled in the medium; during this sampling
c     process, a (black) boundary may be crossed, in which case the event has to stop
            call duplicate_vector(dim,x0,xr)
            scattering_position_found=.false.
            do while (.not.scattering_position_found)
c     Sampling scattering distance from x0
               call choose_tau_unlimited(tau,pdf_tau)
               l_x0x1=tau/ks_hat
c     x1: next intersection position
               call direct_flight(dim,x0,u0,l_x0x1,x1)
c     check x1 inside or outside the surrounding box
               call is_in_box(dim,x1,box_volume,x1_is_in_box)
               if (x1_is_in_box) then
c     update path transmissivity
                  call scattering_coefficient(dim,box_volume,x1,ks)
                  call random_gen(r)
                  if (r.le.ks/ks_hat) then ! scattering @ x1
c     Scattering @ x1
                     scattering_position_found=.true.
                     call null_collision_Tr(dim,box_volume,
     &                    nc_tr_version,xr,x1,'abs',w5)
                     Tr=Tr*w5
                     if (Tr.le.0.0D+0) then
                        keep_going=.false.
                        goto 123
                     endif
                     generate_angle=.true. ! generate scattering angle
                     call choose_scattering_angle(dim,
     &                    generate_angle,
     &                    pf,g_HG,norm_factor,
     &                    alpha,phase_function)
                     call direction_after_scattering(dim,u0,alpha,u1)
                  else          ! null-collision @ x1
                     call duplicate_vector(dim,u0,u1)
                  endif
                  call duplicate_vector(dim,x1,x0)
                  call duplicate_vector(dim,u1,u0)
               else             ! x1_is_in_box=F
c     path reached a boundary of the surrounding box, and is lost
                  keep_going=.false.
                  goto 123
               endif            ! x1_is_in_box
            enddo ! while (.not.scattering_position_found)
 123        continue
         enddo                  ! while (keep_going)
         contrib(1)=pi*Sdetector*weight
c     update of sum and sum2
         do ivar=1,Nvar
            sum(ivar)=sum(ivar)+contrib(ivar)
            sum2(ivar)=sum2(ivar)+contrib(ivar)**2.0D+0
         enddo                  ! ivar
c     ----------------- print status
         ndone=ndone+1
         fdone=dble(ndone)/dble(ntot)*1.0D+2
         ifdone=floor(fdone)
         if (ifdone.gt.pifdone) then
            do j=1,len+2
               write(*,"(a)",advance='no') "\b"
            enddo               ! j
            write(*,trim(fmt),advance='no')
     &           floor(fdone),' %'
            pifdone=ifdone
         endif
c     ----------------- print status
      enddo                     ! event

      do ivar=1,Nvar
         call statistics(Nevent,sum(ivar),sum2(ivar),
     &        mean(ivar),variance(ivar),std_dev(ivar))
      enddo                     ! ivar
      write(*,*)
      write(*,*) 'Reverse MC result:'
      write(*,*) 'Absorbed Flux=',mean(1),' +/-',std_dev(1),' W'
      write(*,*) 'Absorbed Flux density=',mean(1)/Sdetector,
     &     ' +/-',std_dev(1)/Sdetector,' W/m²'
      write(*,*) 'Pa/Pe=',mean(1)/(phi0*S0),
     &     ' +/-',std_dev(1)/(phi0*S0)

      return
      end
